#ifndef XML_OPERT_HEADER
#define XML_OPERT_HEADER

#include <tinyxml2.h>

#include <string>
#include <vector>

#include "config.h"

struct SKeyValue {
    SKeyValue(const char* pkey, const char* pvalue);
    std::string key;
    std::string value;
};

typedef tinyxml2::XMLElement Element_t;
typedef std::vector<SKeyValue> Property_t;
class CXmlOpr
{
public:
    CXmlOpr();
    ~CXmlOpr();

public:
    bool open(const std::string& xml_path);
    bool backup_file(const std::string& desti_folder, const std::string& time);
    void set_baseinfo(const OneGroupIni& base);
    bool parse_xml(std::vector<Element_t*>& vec);
    bool get_all_unit(std::vector<std::string>& units);
    bool get_all_elements(std::vector<Element_t*>& vec,
                          const std::string& unit = "");
    void copy_and_del(std::vector<Element_t*>& vec,
                      std::vector<Element_t*>& out);
    void insert_brother_node(Element_t* brother, Element_t* newer);
    void del_element(Element_t* ele);
    bool check_key_exists(const Property_t& property);
    bool check_key_exists(const std::string& key);
    bool save();
    bool handle_save(const std::string& path, const std::vector<std::string>& keychars);
    std::string handle_space(const std::string& content, const std::vector<std::string>& keychars);

public:
    Element_t* copy_element(Element_t* ele);
    // 检查 xml 格式合法性
    bool check_valid_xml_data(const std::string& data);
    // 不检查 xml 格式合法性，请自行调用 check_valid_xml_data
    bool check_same_struct(const std::string& data);
    // 不检查 xml 格式合法性，请自行调用 check_valid_xml_data
    bool import_newer_data(const std::vector<std::string>& vec,
                           std::size_t& success_count);
    // 处理转义
    bool handle_transfer(const std::string& path, const std::vector<std::string>& keychars);

public:
    void get_attributes(Element_t* ele, Property_t& vec);
    void attributes_to_element(Element_t* ele, const Property_t& vec);

private:
    tinyxml2::XMLDocument doc_{};
    OneGroupIni opr_base_{};
    std::string xml_path_{};
    Element_t* parent_node_{};
    Element_t* parent_node2_{};
    std::vector<std::string> keys_{};
    std::vector<std::string> units_{};
};

#endif
